new;
library pgraph;

proc BlackScholes(S0,K,sigma,tau,r,call_or_put);
  local w,d1,d2,Price;

  w = sigma.*sqrt(tau);
  d1 = ( ln(S0./K) + r.*tau )./w + 0.5 * w;  
  d2 = d1 - w;

  if lower(call_or_put) $== "put";
    Price = -S0.*cdfn(-d1) + K.*exp(-r.*tau).*cdfn(-d2);
  else;
    Price = S0.*cdfn(d1) - K.*exp(-r.*tau).*cdfn(d2);
  endif;

  retp(Price);
endp;


vol = 25/100;          /* Volatilit = 25%                 */
T = 3/12;              /* Maturit = 3 mois                */
S0 = 100;              /* Prix actuel du sous-jacent = 100 */
r = 0.05;

K = S0;                /* Option ATM (at-the-money)        */

BS = BlackScholes(S0,K,vol,T,r,"call");

rndseed 123456;

ns = 1000;

WT = sqrt(T) * rndn(ns,1);
ST = S0 * exp( (r-0.5*vol^2)*T + vol*WT);

PayOff = (ST - K) .* (ST .> K);               /* Payoff = max( S(T) - K, 0) */
Cs = exp(-r*T) * PayOff;

simul = seqa(1,1,ns);
stderr = stdc(Cs);
mcBS = cumsumc(Cs) ./ simul;
mcSTD = stderr ./ sqrt(simul);

graphset;
  _pdate = 0;                     /* Suppression de la date                     */
  _pframe = 0;                    /* Suppression du cadre                       */
  ytics(0,10,2,1);                /* Echelle de l'axes des Y                    */
  _pline = 1~3~0~BS~ns~BS~1~15~5; /* Trait horizontal indiquant le vrai prix BS */

  xy(simul,mcBS + (-1.96~0~1.96) .* mcSTD);




