new;
library pgraph;

Maturity = 5;

N = 100;

Nominal = 100;
Lambda = 50/1e4;
RR = 0.40;

r = 0.03;           // Taux d'intrt
Ns = 1e3;           // Nombre de simulations

Tm = seqa(0.25,0.25,Maturity/0.25);       // Echeancier trimestriel
dTm = 0.25;

B0Tm = exp(-r*Tm);                           // B(t0,tm)

rho = 0.5;
u = cdfn(sqrt(rho)*rndn(1,Ns) + sqrt(1-rho)*rndn(N,Ns));
tau = -ln(1-u)./lambda;

Lower_Tranche = 0.05;
Upper_Tranche = 0.10;

MaxLoss = N * Nominal * (1-RR);
Lower_Tranche = Lower_Tranche * MaxLoss;
Upper_Tranche = Upper_Tranche * MaxLoss;

Loss = zeros(rows(Tm),Ns);
for i(1,Ns,1);
  Loss[.,i] = sumc( Nominal * (1-RR) * (tau[.,i] .<= Tm') );
endfor;

fn _max_(x,y) = x .* (x .> y) + y .* (x .<= y);
fn _min_(x,y) = x .* (x .< y) + y .* (x .>= y);

/*
**> Tranche Equity 
*/

N1 = Lower_Tranche - _min_(Loss,Lower_Tranche);
Loss1 =  missrv(lagn(N1,1),Lower_Tranche) - N1;
JF = sumc(dTm .* B0Tm .* N1);                       
JV = sumc(B0Tm .* Loss1); 
Spread = meanc(JV)/meanc(JF);
print "Spread de la tranche equity : " 1e4*Spread " bp";

/*
**> Tranche Mezzanine
*/

N2 = _max_((Upper_Tranche-Lower_Tranche) - _max_(Loss-Lower_Tranche,0),0);
Loss2 =  missrv(lagn(N2,1),Upper_Tranche-Lower_Tranche) - N2;
JF = sumc(dTm .* B0Tm .* N2);                       
JV = sumc(B0Tm .* Loss2); 
Spread = meanc(JV)/meanc(JF);
print "Spread de la tranche mezzanine : " 1e4*Spread " bp";

/*
**> Tranche Senior
*/

N3 = _max_((maxLoss-Upper_Tranche) - _max_(Loss-Upper_Tranche,0),0);
Loss3 =  missrv(lagn(N3,1),maxLoss-Upper_Tranche) - N3;
JF = sumc(dTm .* B0Tm .* N3);                       
JV = sumc(B0Tm .* Loss3); 
Spread = meanc(JV)/meanc(JF);
print "Spread de la tranche senior : " 1e4*Spread " bp";


/*
**> Tranche Mezzanine
*/

N4 = _max_((Upper_Tranche-Lower_Tranche) - _max_(Loss-Lower_Tranche,0),0);
Loss4 =  missrv(lagn(N2,1),Upper_Tranche-Lower_Tranche) - N2;

Loss2 =  _min_(_max_(Loss-Lower_Tranche,0),Upper_Tranche-Lower_Tranche);
N2 = (Upper_Tranche-Lower_Tranche) - Loss2;
JF = sumc(dTm .* B0Tm .* N2);                       
JV = sumc(B0Tm .* Loss2); 
Spread = meanc(JV)/meanc(JF);
print "Spread de la tranche mezzanine : " 1e4*Spread " bp";



