/*
**> Estimation de pertes selon une distribution log-normale
**  lorsqu'il y a un threshold
**
*/

new;
library pgraph;

rndseed 123;          /* Simulation des memes pertes */

mu = 7;               /* Paramtres de la svrit */
sigma = 1.75;

Ns = 10000;

u = mu + sigma*rndn(Ns,1);     /* Simulations de va N(mu,sigma)  */
Losses = exp(u);               /* Simulations de va LN(mu,sigma) */

Threshold = 1000;              /* Seuil de collecte              */
Losses_obs = selif(Losses, Losses .>= Threshold);   /* Pertes collectees */

/*
**> Estimation classique
**
*/

data = Losses_obs;

m1 = meanc(ln(data));
s1 = stdc(ln(data));

/*
*> Estimation par maximum de vraisemblance sans correction
**
*/

proc mlProc(theta);
  local mu,sigma,LogL;

  theta = sqrt(theta^2);     /* Pour tre sr que les paramtres sont positifs */
  mu = theta[1];
  sigma = theta[2];

  /* Vecteur des fonctions de vraisemblance */
  LogL = -0.5*ln(sigma^2) - 0.5*ln(2*pi) - ln(data) - 0.5*(ln(data)-mu)^2 ./ sigma^2;

  retp(Logl);
endp;

fn negLogL(theta) = -sumc(mlProc(theta));

_qn_PrintIters = 1;
_qn_RelGradTol = 1e-5;

sv = mu | sigma;
{theta_opt,fmin,grd,retcode} = Qnewton(&negLogL,sv);
theta_opt = abs(theta_opt);

m2 = theta_opt[1];
s2 = theta_opt[2];

print mu~m1~m2;
print sigma~s1~s2;