new;

proc EuropeanBS(S0,K,sigma,T,r);
  local w,d1,d2,P;

  w = sigma.*sqrt(T);
  d1 = ( ln(S0./K) + r.*T )./w + 0.5 * w;  
  d2 = d1 - w;

  P = S0.*cdfn(d1) - K.*exp(-r.*T).*cdfn(d2);
  retp(P);
endp;

load Prices[251,3] = prix.dat;
S = Prices[.,1];
r = packr( ln(S) - lag1(ln(S)) );

S_t = 234.31;

K = 220;
tau = 90/365;
ir = 0.05;
sigma_t = 0.237;
C_t = EuropeanBS(S_t,K,sigma_t,tau,ir);
shock = 1e-4;
Delta_t = (EuropeanBS(S_t*(1+shock),K,sigma_t,tau,ir) - EuropeanBS(S_t,K,sigma_t,tau,ir))/(shock*S_t);

theta1 = -1; theta2 = 10;                       // Composition du portefeuille
P_t = theta1*S_t + theta2*C_t;                  // valeur actuelle du portefeuille

/*
**> VaR historique (le facteur de risque est la rendement de l'actif)
**
*/

S_tp1 = S_t .* (1+r);
C_tp1 = EuropeanBS(S_tp1,K,sigma_t,tau-1/365,ir);
P_tp1 = theta1*S_tp1 + theta2*C_tp1;
PnL = P_tp1 - P_t; 
VaR_histo = -quantile(PnL,0.01);
print VaR_histo;

/*
**> VaR monte carlo
**
*/

n = 100000;
mu_r = meanc(r);
sigma_r = stdc(r);
rs = mu_r + rndn(n,1) * sigma_r;
S_tp1 = S_t .* (1+rs);
C_tp1 = EuropeanBS(S_tp1,K,sigma_t,tau-1/365,ir);
P_tp1 = theta1*S_tp1 + theta2*C_tp1;
PnL_MC = P_tp1 - P_t; 
VaR_MC = -quantile(PnL_MC,0.01);
print VaR_MC;

/*
**> VaR delta
**
*/

PnL_Delta = (theta1+theta2*Delta_t)*S_t.*r; 
VaR_Delta = -quantile(PnL_Delta,0.01);
print VaR_Delta;

